@@ -2,12 +2,12 @@ from __future__ import annotations
 
 import contextlib
 from typing import Any
-from typing import Callable
 from typing import Dict
 from typing import Iterator
+from typing import List
 from typing import Optional
+from typing import Sequence
 from typing import Set
-from typing import Tuple
 from typing import TYPE_CHECKING
 from typing import Union
 
@@ -17,6 +17,7 @@ from . import compare
 from . import render
 from .. import util
 from ..operations import ops
+from ..util import sqla_compat
 
 """Provide the 'autogenerate' feature which can produce migration operations
 automatically."""
@@ -25,22 +26,25 @@ if TYPE_CHECKING:
     from sqlalchemy.engine import Connection
     from sqlalchemy.engine import Dialect
     from sqlalchemy.engine import Inspector
-    from sqlalchemy.sql.schema import Column
-    from sqlalchemy.sql.schema import ForeignKeyConstraint
-    from sqlalchemy.sql.schema import Index
     from sqlalchemy.sql.schema import MetaData
+    from sqlalchemy.sql.schema import SchemaItem
     from sqlalchemy.sql.schema import Table
-    from sqlalchemy.sql.schema import UniqueConstraint
 
-    from alembic.config import Config
-    from alembic.operations.ops import MigrationScript
-    from alembic.operations.ops import UpgradeOps
-    from alembic.runtime.migration import MigrationContext
-    from alembic.script.base import Script
-    from alembic.script.base import ScriptDirectory
+    from ..config import Config
+    from ..operations.ops import DowngradeOps
+    from ..operations.ops import MigrationScript
+    from ..operations.ops import UpgradeOps
+    from ..runtime.environment import NameFilterParentNames
+    from ..runtime.environment import NameFilterType
+    from ..runtime.environment import ProcessRevisionDirectiveFn
+    from ..runtime.environment import RenderItemFn
+    from ..runtime.migration import MigrationContext
+    from ..script.base import Script
+    from ..script.base import ScriptDirectory
+    from ..script.revision import _GetRevArg
 
 
-def compare_metadata(context: "MigrationContext", metadata: "MetaData") -> Any:
+def compare_metadata(context: MigrationContext, metadata: MetaData) -> Any:
     """Compare a database schema to that given in a
     :class:`~sqlalchemy.schema.MetaData` instance.
 
@@ -57,36 +61,42 @@ def compare_metadata(context: "MigrationContext", metadata: "MetaData") -> Any:
 
         from alembic.migration import MigrationContext
         from alembic.autogenerate import compare_metadata
-        from sqlalchemy.schema import SchemaItem
-        from sqlalchemy.types import TypeEngine
-        from sqlalchemy import (create_engine, MetaData, Column,
-                Integer, String, Table, text)
+        from sqlalchemy import (
+            create_engine,
+            MetaData,
+            Column,
+            Integer,
+            String,
+            Table,
+            text,
+        )
         import pprint
 
         engine = create_engine("sqlite://")
 
         with engine.begin() as conn:
-            conn.execute(text('''
-                create table foo (
-                    id integer not null primary key,
-                    old_data varchar,
-                    x integer
-                )'''))
-
-            conn.execute(text('''
-                create table bar (
-                    data varchar
-                )'''))
+            conn.execute(
+                text(
+                    '''
+                        create table foo (
+                            id integer not null primary key,
+                            old_data varchar,
+                            x integer
+                        )
+                    '''
+                )
+            )
+            conn.execute(text("create table bar (data varchar)"))
 
         metadata = MetaData()
-        Table('foo', metadata,
-            Column('id', Integer, primary_key=True),
-            Column('data', Integer),
-            Column('x', Integer, nullable=False)
-        )
-        Table('bat', metadata,
-            Column('info', String)
+        Table(
+            "foo",
+            metadata,
+            Column("id", Integer, primary_key=True),
+            Column("data", Integer),
+            Column("x", Integer, nullable=False),
         )
+        Table("bat", metadata, Column("info", String))
 
         mc = MigrationContext.configure(engine.connect())
 
@@ -95,29 +105,53 @@ def compare_metadata(context: "MigrationContext", metadata: "MetaData") -> Any:
 
     Output::
 
-        [ ( 'add_table',
-            Table('bat', MetaData(bind=None),
-                Column('info', String(), table=<bat>), schema=None)),
-          ( 'remove_table',
-            Table(u'bar', MetaData(bind=None),
-                Column(u'data', VARCHAR(), table=<bar>), schema=None)),
-          ( 'add_column',
-            None,
-            'foo',
-            Column('data', Integer(), table=<foo>)),
-          ( 'remove_column',
-            None,
-            'foo',
-            Column(u'old_data', VARCHAR(), table=None)),
-          [ ( 'modify_nullable',
-              None,
-              'foo',
-              u'x',
-              { 'existing_server_default': None,
-                'existing_type': INTEGER()},
-              True,
-              False)]]
-
+        [
+            (
+                "add_table",
+                Table(
+                    "bat",
+                    MetaData(),
+                    Column("info", String(), table=<bat>),
+                    schema=None,
+                ),
+            ),
+            (
+                "remove_table",
+                Table(
+                    "bar",
+                    MetaData(),
+                    Column("data", VARCHAR(), table=<bar>),
+                    schema=None,
+                ),
+            ),
+            (
+                "add_column",
+                None,
+                "foo",
+                Column("data", Integer(), table=<foo>),
+            ),
+            [
+                (
+                    "modify_nullable",
+                    None,
+                    "foo",
+                    "x",
+                    {
+                        "existing_comment": None,
+                        "existing_server_default": False,
+                        "existing_type": INTEGER(),
+                    },
+                    True,
+                    False,
+                )
+            ],
+            (
+                "remove_column",
+                None,
+                "foo",
+                Column("old_data", VARCHAR(), table=<foo>),
+            ),
+        ]
 
     :param context: a :class:`.MigrationContext`
      instance.
@@ -132,12 +166,13 @@ def compare_metadata(context: "MigrationContext", metadata: "MetaData") -> Any:
     """
 
     migration_script = produce_migrations(context, metadata)
+    assert migration_script.upgrade_ops is not None
     return migration_script.upgrade_ops.as_diffs()
 
 
 def produce_migrations(
-    context: "MigrationContext", metadata: "MetaData"
-) -> "MigrationScript":
+    context: MigrationContext, metadata: MetaData
+) -> MigrationScript:
     """Produce a :class:`.MigrationScript` structure based on schema
     comparison.
 
@@ -167,13 +202,14 @@ def produce_migrations(
 
 
 def render_python_code(
-    up_or_down_op: "UpgradeOps",
+    up_or_down_op: Union[UpgradeOps, DowngradeOps],
     sqlalchemy_module_prefix: str = "sa.",
     alembic_module_prefix: str = "op.",
     render_as_batch: bool = False,
-    imports: Tuple[str, ...] = (),
-    render_item: None = None,
-    migration_context: Optional["MigrationContext"] = None,
+    imports: Sequence[str] = (),
+    render_item: Optional[RenderItemFn] = None,
+    migration_context: Optional[MigrationContext] = None,
+    user_module_prefix: Optional[str] = None,
 ) -> str:
     """Render Python code given an :class:`.UpgradeOps` or
     :class:`.DowngradeOps` object.
@@ -181,12 +217,24 @@ def render_python_code(
     This is a convenience function that can be used to test the
     autogenerate output of a user-defined :class:`.MigrationScript` structure.
 
+    :param up_or_down_op: :class:`.UpgradeOps` or :class:`.DowngradeOps` object
+    :param sqlalchemy_module_prefix: module prefix for SQLAlchemy objects
+    :param alembic_module_prefix: module prefix for Alembic constructs
+    :param render_as_batch: use "batch operations" style for rendering
+    :param imports: sequence of import symbols to add
+    :param render_item: callable to render items
+    :param migration_context: optional :class:`.MigrationContext`
+    :param user_module_prefix: optional string prefix for user-defined types
+
+     .. versionadded:: 1.11.0
+
     """
     opts = {
         "sqlalchemy_module_prefix": sqlalchemy_module_prefix,
         "alembic_module_prefix": alembic_module_prefix,
         "render_item": render_item,
         "render_as_batch": render_as_batch,
+        "user_module_prefix": user_module_prefix,
     }
 
     if migration_context is None:
@@ -205,7 +253,7 @@ def render_python_code(
 
 
 def _render_migration_diffs(
-    context: "MigrationContext", template_args: Dict[Any, Any]
+    context: MigrationContext, template_args: Dict[Any, Any]
 ) -> None:
     """legacy, used by test_autogen_composition at the moment"""
 
@@ -229,7 +277,7 @@ class AutogenContext:
     """Maintains configuration and state that's specific to an
     autogenerate operation."""
 
-    metadata: Optional["MetaData"] = None
+    metadata: Optional[MetaData] = None
     """The :class:`~sqlalchemy.schema.MetaData` object
     representing the destination.
 
@@ -247,7 +295,7 @@ class AutogenContext:
 
     """
 
-    connection: Optional["Connection"] = None
+    connection: Optional[Connection] = None
     """The :class:`~sqlalchemy.engine.base.Connection` object currently
     connected to the database backend being compared.
 
@@ -256,7 +304,7 @@ class AutogenContext:
 
     """
 
-    dialect: Optional["Dialect"] = None
+    dialect: Optional[Dialect] = None
     """The :class:`~sqlalchemy.engine.Dialect` object currently in use.
 
     This is normally obtained from the
@@ -278,17 +326,16 @@ class AutogenContext:
 
     """
 
-    migration_context: "MigrationContext" = None  # type: ignore[assignment]
+    migration_context: MigrationContext = None  # type: ignore[assignment]
     """The :class:`.MigrationContext` established by the ``env.py`` script."""
 
     def __init__(
         self,
-        migration_context: "MigrationContext",
-        metadata: Optional["MetaData"] = None,
-        opts: Optional[dict] = None,
+        migration_context: MigrationContext,
+        metadata: Optional[MetaData] = None,
+        opts: Optional[Dict[str, Any]] = None,
         autogenerate: bool = True,
     ) -> None:
-
         if (
             autogenerate
             and migration_context is not None
@@ -342,7 +389,7 @@ class AutogenContext:
         self._has_batch: bool = False
 
     @util.memoized_property
-    def inspector(self) -> "Inspector":
+    def inspector(self) -> Inspector:
         if self.connection is None:
             raise TypeError(
                 "can't return inspector as this "
@@ -359,8 +406,8 @@ class AutogenContext:
     def run_name_filters(
         self,
         name: Optional[str],
-        type_: str,
-        parent_names: Dict[str, Optional[str]],
+        type_: NameFilterType,
+        parent_names: NameFilterParentNames,
     ) -> bool:
         """Run the context's name filters and return True if the targets
         should be part of the autogenerate operation.
@@ -388,7 +435,6 @@ class AutogenContext:
                     parent_names["schema_qualified_table_name"] = table_name
 
         for fn in self._name_filters:
-
             if not fn(name, type_, parent_names):
                 return False
         else:
@@ -396,19 +442,11 @@ class AutogenContext:
 
     def run_object_filters(
         self,
-        object_: Union[
-            "Table",
-            "Index",
-            "Column",
-            "UniqueConstraint",
-            "ForeignKeyConstraint",
-        ],
-        name: Optional[str],
-        type_: str,
+        object_: SchemaItem,
+        name: sqla_compat._ConstraintName,
+        type_: NameFilterType,
         reflected: bool,
-        compare_to: Optional[
-            Union["Table", "Index", "Column", "UniqueConstraint"]
-        ],
+        compare_to: Optional[SchemaItem],
     ) -> bool:
         """Run the context's object filters and return True if the targets
         should be part of the autogenerate operation.
@@ -429,8 +467,9 @@ class AutogenContext:
     run_filters = run_object_filters
 
     @util.memoized_property
-    def sorted_tables(self):
-        """Return an aggregate of the :attr:`.MetaData.sorted_tables` collection(s).
+    def sorted_tables(self) -> List[Table]:
+        """Return an aggregate of the :attr:`.MetaData.sorted_tables`
+        collection(s).
 
         For a sequence of :class:`.MetaData` objects, this
         concatenates the :attr:`.MetaData.sorted_tables` collection
@@ -444,7 +483,7 @@ class AutogenContext:
         return result
 
     @util.memoized_property
-    def table_key_to_table(self):
+    def table_key_to_table(self) -> Dict[str, Table]:
         """Return an aggregate  of the :attr:`.MetaData.tables` dictionaries.
 
         The :attr:`.MetaData.tables` collection is a dictionary of table key
@@ -455,7 +494,7 @@ class AutogenContext:
         objects contain the same table key, an exception is raised.
 
         """
-        result = {}
+        result: Dict[str, Table] = {}
         for m in util.to_list(self.metadata):
             intersect = set(result).intersection(set(m.tables))
             if intersect:
@@ -473,12 +512,17 @@ class RevisionContext:
     """Maintains configuration and state that's specific to a revision
     file generation operation."""
 
+    generated_revisions: List[MigrationScript]
+    process_revision_directives: Optional[ProcessRevisionDirectiveFn]
+
     def __init__(
         self,
-        config: "Config",
-        script_directory: "ScriptDirectory",
+        config: Config,
+        script_directory: ScriptDirectory,
         command_args: Dict[str, Any],
-        process_revision_directives: Optional[Callable] = None,
+        process_revision_directives: Optional[
+            ProcessRevisionDirectiveFn
+        ] = None,
     ) -> None:
         self.config = config
         self.script_directory = script_directory
@@ -491,8 +535,8 @@ class RevisionContext:
         self.generated_revisions = [self._default_revision()]
 
     def _to_script(
-        self, migration_script: "MigrationScript"
-    ) -> Optional["Script"]:
+        self, migration_script: MigrationScript
+    ) -> Optional[Script]:
         template_args: Dict[str, Any] = self.template_args.copy()
 
         if getattr(migration_script, "_needs_render", False):
@@ -521,21 +565,21 @@ class RevisionContext:
         )
 
     def run_autogenerate(
-        self, rev: tuple, migration_context: "MigrationContext"
-    ):
+        self, rev: _GetRevArg, migration_context: MigrationContext
+    ) -> None:
         self._run_environment(rev, migration_context, True)
 
     def run_no_autogenerate(
-        self, rev: tuple, migration_context: "MigrationContext"
-    ):
+        self, rev: _GetRevArg, migration_context: MigrationContext
+    ) -> None:
         self._run_environment(rev, migration_context, False)
 
     def _run_environment(
         self,
-        rev: tuple,
-        migration_context: "MigrationContext",
+        rev: _GetRevArg,
+        migration_context: MigrationContext,
         autogenerate: bool,
-    ):
+    ) -> None:
         if autogenerate:
             if self.command_args["sql"]:
                 raise util.CommandError(
@@ -552,9 +596,9 @@ class RevisionContext:
         migration_script = self.generated_revisions[-1]
         if not getattr(migration_script, "_needs_render", False):
             migration_script.upgrade_ops_list[-1].upgrade_token = upgrade_token
-            migration_script.downgrade_ops_list[
-                -1
-            ].downgrade_token = downgrade_token
+            migration_script.downgrade_ops_list[-1].downgrade_token = (
+                downgrade_token
+            )
             migration_script._needs_render = True
         else:
             migration_script._upgrade_ops.append(
@@ -586,7 +630,7 @@ class RevisionContext:
         for migration_script in self.generated_revisions:
             migration_script._needs_render = True
 
-    def _default_revision(self) -> "MigrationScript":
+    def _default_revision(self) -> MigrationScript:
         command_args: Dict[str, Any] = self.command_args
         op = ops.MigrationScript(
             rev_id=command_args["rev_id"] or util.rev_id(),
@@ -601,6 +645,6 @@ class RevisionContext:
         )
         return op
 
-    def generate_scripts(self) -> Iterator[Optional["Script"]]:
+    def generate_scripts(self) -> Iterator[Optional[Script]]:
         for generated_revision in self.generated_revisions:
             yield self._to_script(generated_revision)
