@@ -406,8 +406,7 @@ class AbstractOperations(util.ModuleClsProxy):
         return self.migration_context
 
     @overload
-    def invoke(self, operation: CreateTableOp) -> Table:
-        ...
+    def invoke(self, operation: CreateTableOp) -> Table: ...
 
     @overload
     def invoke(
@@ -427,12 +426,10 @@ class AbstractOperations(util.ModuleClsProxy):
             DropTableOp,
             ExecuteSQLOp,
         ],
-    ) -> None:
-        ...
+    ) -> None: ...
 
     @overload
-    def invoke(self, operation: MigrateOperation) -> Any:
-        ...
+    def invoke(self, operation: MigrateOperation) -> Any: ...
 
     def invoke(self, operation: MigrateOperation) -> Any:
         """Given a :class:`.MigrateOperation`, invoke it in terms of
@@ -1724,7 +1721,7 @@ class BatchOperations(AbstractOperations):
 
         def create_foreign_key(
             self,
-            constraint_name: str,
+            constraint_name: Optional[str],
             referent_table: str,
             local_cols: List[str],
             remote_cols: List[str],
@@ -1774,7 +1771,7 @@ class BatchOperations(AbstractOperations):
             ...
 
         def create_primary_key(
-            self, constraint_name: str, columns: List[str]
+            self, constraint_name: Optional[str], columns: List[str]
         ) -> None:
             """Issue a "create primary key" instruction using the
             current batch migration context.
