@@ -1,4 +1,5 @@
 from sqlalchemy import exc as sqla_exc
+from sqlalchemy import Index
 from sqlalchemy import text
 
 from alembic.testing import exclusions
@@ -138,8 +139,14 @@ class DefaultRequirements(SuiteRequirements):
     def reflects_indexes_w_sorting(self):
         # TODO: figure out what's happening on the SQLAlchemy side
         # when we reflect an index that has asc() / desc() on the column
+        # Tracked by https://github.com/sqlalchemy/sqlalchemy/issues/9597
         return exclusions.fails_on(["oracle"])
 
+    @property
+    def reflects_indexes_column_sorting(self):
+        "Actually reflect column_sorting on the indexes"
+        return exclusions.only_on(["postgresql"])
+
     @property
     def long_names(self):
         if sqla_compat.sqla_14:
@@ -377,10 +384,6 @@ class DefaultRequirements(SuiteRequirements):
             ["postgresql >= 10", "oracle >= 12"]
         )
 
-    @property
-    def supports_identity_on_null(self):
-        return self.identity_columns + exclusions.only_on(["oracle"])
-
     @property
     def legacy_engine(self):
         return exclusions.only_if(
@@ -390,9 +393,12 @@ class DefaultRequirements(SuiteRequirements):
     @property
     def stubs_test(self):
         def requirements():
+            import warnings
+
             try:
-                import black  # noqa
-                import zimports  # noqa
+                with warnings.catch_warnings(action="ignore"):
+                    import black  # noqa
+                    import zimports  # noqa
 
                 return False
             except Exception:
@@ -402,7 +408,7 @@ class DefaultRequirements(SuiteRequirements):
             requirements, "black and zimports are required for this test"
         )
         version = exclusions.only_if(
-            lambda _: compat.py39, "python 3.9 is required"
+            lambda _: compat.py311, "python 3.11 is required"
         )
 
         sqlalchemy = exclusions.only_if(
@@ -410,3 +416,37 @@ class DefaultRequirements(SuiteRequirements):
         )
 
         return imports + version + sqlalchemy
+
+    @property
+    def reflect_indexes_with_expressions(self):
+        sqlalchemy = exclusions.only_if(
+            lambda _: sqla_compat.sqla_2, "sqlalchemy 2 is required"
+        )
+
+        postgresql = exclusions.only_on(["postgresql"])
+
+        return sqlalchemy + postgresql
+
+    @property
+    def indexes_with_expressions(self):
+        return exclusions.only_on(["postgresql", "sqlite>=3.9.0"])
+
+    @property
+    def nulls_not_distinct_sa(self):
+        def _has_nulls_not_distinct():
+            try:
+                Index("foo", "bar", postgresql_nulls_not_distinct=True)
+                return True
+            except sqla_exc.ArgumentError:
+                return False
+
+        return exclusions.only_if(
+            _has_nulls_not_distinct,
+            "sqlalchemy with nulls not distinct support needed",
+        )
+
+    @property
+    def nulls_not_distinct_db(self):
+        return self.nulls_not_distinct_sa + exclusions.only_on(
+            ["postgresql>=15"]
+        )
